#!/usr/bin/perl
#
# Sensor for gathering hardware data for Zabbix via
# IPMI.
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Tue Mar 17 18:04:04 CET 2009
# 	- rewritten sender to use only one connection
# 	- fixed alarms
# * Thu May 15 03:40:39 CEST 2008       
#       - script created
#

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileIPMI";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = '';
my $hostname = `/bin/hostname -f`;
chomp($hostname);
my $ipmitool="/usr/bin/ipmitool";

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

# fetch status from mod_status
alarm 25;
my $cmd = "sudo $ipmitool sdr list";
my $server_status = qx($cmd);

# get information from status output
my($ChassisTemp, $CPU1temp,$CPU2temp,$CPU3temp,$CPU4temp, $Fan1,$Fan2,$Fan3,$Fan4, $Fan5, $Fan6);
$ChassisTemp = $1 if ($server_status =~ /Ambient Temp \s+\|\s+([\d]+)\s+degrees/);
$CPU1temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU2temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU3temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$CPU4temp = $1 if ($server_status =~ s/Temp \s+\|\s+([\d]+)\s+degrees//);
$Fan1 = $1 if ($server_status =~ /FAN 1 .+\|\s+([\d]+)\s+RPM/);
$Fan2 = $1 if ($server_status =~ /FAN 2 .+\|\s+([\d]+)\s+RPM/);
$Fan3 = $1 if ($server_status =~ /FAN 3 .+\|\s+([\d]+)\s+RPM/);
$Fan4 = $1 if ($server_status =~ /FAN 4 .+\|\s+([\d]+)\s+RPM/);
$Fan5 = $1 if ($server_status =~ /FAN 5 .+\|\s+([\d]+)\s+RPM/);
$Fan6 = $1 if ($server_status =~ /FAN 6 .+\|\s+([\d]+)\s+RPM/);
#print $ChassisTemp."\n";

# send everything, if sending fails exit with zero
$inputString .= $hostname ." ipmi.temp.cpu1 ".	$CPU1temp	."\n" if($CPU1temp);
$inputString .= $hostname ." ipmi.temp.cpu2 ".	$CPU2temp	."\n" if($CPU2temp);
$inputString .= $hostname ." ipmi.temp.cpu3 ".	$CPU3temp	."\n" if($CPU3temp);
$inputString .= $hostname ." ipmi.temp.cpu4 ".	$CPU4temp	."\n" if($CPU4temp);
$inputString .= $hostname ." ipmi.temp.chassis ". $ChassisTemp	."\n" if($ChassisTemp);
$inputString .= $hostname ." ipmi.fan.1 ".	$Fan1		."\n" if($Fan1);
$inputString .= $hostname ." ipmi.fan.2 ".	$Fan2		."\n" if($Fan2);
$inputString .= $hostname ." ipmi.fan.3 ".	$Fan3		."\n" if($Fan3);
$inputString .= $hostname ." ipmi.fan.4 ".	$Fan4		."\n" if($Fan4);
$inputString .= $hostname ." ipmi.fan.5 ".	$Fan5		."\n" if($Fan5);
$inputString .= $hostname ." ipmi.fan.6 ".	$Fan6		."\n" if($Fan6);

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
#EOF
